@@ -1,8 +1,55 @@
 from fabric.api import *
 from fabric.contrib import django
+from fabric.contrib.console import confirm
 
 
-def deploy(branch, plan):
+def full_deploy(branch):
+  django.settings_module(env.django_settings_module)
+  from django.conf import settings
+
+  project_name = env.project_name
+  project_path = '/var/www/%s' % project_name
+  project_settings_path = '/var/www/.settings/%s.py' % project_name
+  sites_path = '/var/www/.sites/'
+  media_path = '/var/www/.media/%s' % project_name
+  script_path = '/var/www/.scripts/%s.sh' % project_name
+  system_now = run('date +\%Y\%m\%d\%H\%M\%S')
+  new_instance_path = sites_path + project_name + "_" + system_now
+
+  with cd(sites_path):
+    existing_instances = run('ls|grep "%s_"' % project_name).split('\n')
+
+  run("git clone %s %s" % (env.repo_source, new_instance_path))
+
+  with cd(new_instance_path):
+    run("git fetch origin")
+    run("git checkout %s" % branch)
+    run("git pull")
+    run("git submodule init")
+    run("git submodule update")
+    run("virtualenv env")
+    run(
+      "ln -sf %s %s/%s/settings/__init__.py" % (
+        project_settings_path, new_instance_path, project_name))
+    with prefix('source %s/env/bin/activate' % new_instance_path):
+      run("pip install --upgrade -r requirements.txt --use-mirrors")
+      run("ln -s /usr/lib/python2.7/dist-packages/xapian/ "
+          "env/lib/python2.7/site-packages/.")
+      run("python manage.py migrate")
+      run("python manage.py collectstatic --noinput")
+      if 'haystack' in settings.INSTALLED_APPS:
+        run("python manage.py rebuild_index --noinput")
+
+  run("ln -sf %s %s/media" % (media_path, new_instance_path))
+  run("ln -sfn %s %s" % (new_instance_path, project_path))
+
+  run("source %s" % script_path)
+
+  for old_instance in existing_instances:
+    run("rm -rf %s%s" % (sites_path, old_instance))
+
+
+def fast_deploy(branch):
   django.settings_module(env.django_settings_module)
   from django.conf import settings
 
@@ -17,12 +64,19 @@ def deploy(branch, plan):
       run("git pull")
       run("git submodule init")
       run("git submodule update")
-      if plan == 'full':
-        run("pip install --upgrade -r requirements.txt --use-mirrors")
+
       run("python manage.py migrate")
       if 'haystack' in settings.INSTALLED_APPS:
         run("python manage.py rebuild_index --noinput")
       run("python manage.py collectstatic --noinput")
+
+
+def deploy(branch, plan):
+  if plan == 'fast':
+    fast_deploy(branch)
+  elif plan == 'full':
+    full_deploy(branch)
+
   sudo("service apache2 restart", shell=False)
   sudo("service memcached restart", shell=False)
   sudo("service nginx restart", shell=False)
@@ -54,13 +108,9 @@ def get_deploy_role():
 
 def run_deploy(plan):
   if env.deploy_role == 'staging':
-    if plan == 'full':
-      test_branch('master')
     execute(deploy, 'master', plan, role='staging')
   elif env.deploy_role == 'production':
-    if plan == 'full':
-      test_branch('production')
-    elif plan == 'fast':
+    if plan == 'fast':
       if not confirm(
         "Fast deploys to production are discouraged.  Continue anyway?"):
         abort("Production fast deploy cancelled.")
